#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-08-30 17:21:13

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from dataclasses import dataclass, field
from enum import Enum
from typing import List, Optional

from xsdata.models.datatype import XmlDateTime

__NAMESPACE__ = "urn:SICD:1.3.0"


@dataclass
class ArrayDouble:
    value: Optional[float] = field(
        default=None,
        metadata={
            "required": True,
        },
    )
    index: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


class CollectType(Enum):
    MONOSTATIC = "MONOSTATIC"
    BISTATIC = "BISTATIC"


@dataclass
class ComplexType:
    real: Optional[float] = field(
        default=None,
        metadata={
            "name": "Real",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    imag: Optional[float] = field(
        default=None,
        metadata={
            "name": "Imag",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


class CornerStringType(Enum):
    FRFC_1 = "1:FRFC"
    FRLC_2 = "2:FRLC"
    LRLC_3 = "3:LRLC"
    LRFC_4 = "4:LRFC"


class DirParamTypeSgn(Enum):
    PLUS_ONE = 1
    MINUS_ONE = -1


@dataclass
class ErrorDecorrFuncType:
    corr_coef_zero: Optional[float] = field(
        default=None,
        metadata={
            "name": "CorrCoefZero",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    decorr_rate: Optional[float] = field(
        default=None,
        metadata={
            "name": "DecorrRate",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


class EarthModel(Enum):
    WGS_84 = "WGS_84"


class ImagePlane(Enum):
    SLANT = "SLANT"
    GROUND = "GROUND"
    OTHER = "OTHER"


class ImageGridType(Enum):
    RGAZIM = "RGAZIM"
    RGZERO = "RGZERO"
    XRGYCR = "XRGYCR"
    XCTYAT = "XCTYAT"
    PLANE = "PLANE"


@dataclass
class ImageCreationType:
    application: Optional[str] = field(
        default=None,
        metadata={
            "name": "Application",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    date_time: Optional[XmlDateTime] = field(
        default=None,
        metadata={
            "name": "DateTime",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    site: Optional[str] = field(
        default=None,
        metadata={
            "name": "Site",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    profile: Optional[str] = field(
        default=None,
        metadata={
            "name": "Profile",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )


class ImagePixelType(Enum):
    RE32F_IM32F = "RE32F_IM32F"
    RE16I_IM16I = "RE16I_IM16I"
    AMP8I_PHS8I = "AMP8I_PHS8I"


class AzAutofocus(Enum):
    NO = "NO"
    GLOBAL = "GLOBAL"
    SV = "SV"


class ImageBeamComp(Enum):
    NO = "NO"
    SV = "SV"


class ImageFormAlgo(Enum):
    PFA = "PFA"
    RMA = "RMA"
    RGAZCOMP = "RGAZCOMP"
    OTHER = "OTHER"


class RgAutofocus(Enum):
    NO = "NO"
    GLOBAL = "GLOBAL"
    SV = "SV"


class STBeamComp(Enum):
    NO = "NO"
    GLOBAL = "GLOBAL"
    SV = "SV"


@dataclass
class LatLonHAERestrictionType:
    lat: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lat",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": -90.0,
            "max_inclusive": 90.0,
        },
    )
    lon: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lon",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": -180.0,
            "max_inclusive": 180.0,
        },
    )
    hae: Optional[float] = field(
        default=None,
        metadata={
            "name": "HAE",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class LatLonHAEType:
    lat: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lat",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    lon: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lon",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    hae: Optional[float] = field(
        default=None,
        metadata={
            "name": "HAE",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class LatLonRestrictionType:
    lat: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lat",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": -90.0,
            "max_inclusive": 90.0,
        },
    )
    lon: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lon",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": -180.0,
            "max_inclusive": 180.0,
        },
    )


@dataclass
class LatLonType:
    lat: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lat",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    lon: Optional[float] = field(
        default=None,
        metadata={
            "name": "Lon",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


class NoiseLevelType(Enum):
    ABSOLUTE = "ABSOLUTE"
    RELATIVE = "RELATIVE"


class OrientationType(Enum):
    UP = "UP"
    DOWN = "DOWN"
    LEFT = "LEFT"
    RIGHT = "RIGHT"
    ARBITRARY = "ARBITRARY"


@dataclass
class ParameterType:
    value: str = field(
        default="",
        metadata={
            "required": True,
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class PolyCoef1DType:
    value: Optional[float] = field(
        default=None,
        metadata={
            "required": True,
        },
    )
    exponent1: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class PolyCoef2DType:
    value: Optional[float] = field(
        default=None,
        metadata={
            "required": True,
        },
    )
    exponent1: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    exponent2: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


class PosVelErrFrame(Enum):
    ECF = "ECF"
    RIC_ECF = "RIC_ECF"
    RIC_ECI = "RIC_ECI"


class RMAImageType(Enum):
    RMAT = "RMAT"
    RMCR = "RMCR"
    INCA = "INCA"


class RMAlgorithm(Enum):
    OMEGA_K = "OMEGA_K"
    CSA = "CSA"
    RG_DOP = "RG_DOP"


class RadarModeType(Enum):
    SPOTLIGHT = "SPOTLIGHT"
    STRIPMAP = "STRIPMAP"
    DYNAMIC_STRIPMAP = "DYNAMIC STRIPMAP"


@dataclass
class RowColType:
    row: Optional[int] = field(
        default=None,
        metadata={
            "name": "Row",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    col: Optional[int] = field(
        default=None,
        metadata={
            "name": "Col",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


class SideOfTrack(Enum):
    L = "L"
    R = "R"


class WFParametersRcvDemodType(Enum):
    STRETCH = "STRETCH"
    CHIRP = "CHIRP"


@dataclass
class XYZType:
    x: Optional[float] = field(
        default=None,
        metadata={
            "name": "X",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    y: Optional[float] = field(
        default=None,
        metadata={
            "name": "Y",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    z: Optional[float] = field(
        default=None,
        metadata={
            "name": "Z",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class CollectionInfoType:
    collector_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "CollectorName",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    illuminator_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "IlluminatorName",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    core_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "CoreName",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    collect_type: Optional[CollectType] = field(
        default=None,
        metadata={
            "name": "CollectType",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    radar_mode: Optional["CollectionInfoType.RadarMode"] = field(
        default=None,
        metadata={
            "name": "RadarMode",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    classification: Optional[str] = field(
        default=None,
        metadata={
            "name": "Classification",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    country_code: List[str] = field(
        default_factory=list,
        metadata={
            "name": "CountryCode",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    parameter: List[ParameterType] = field(
        default_factory=list,
        metadata={
            "name": "Parameter",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class RadarMode:
        mode_type: Optional[RadarModeType] = field(
            default=None,
            metadata={
                "name": "ModeType",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        mode_id: Optional[str] = field(
            default=None,
            metadata={
                "name": "ModeID",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )


@dataclass
class ErrorStatisticsType:
    composite_scp: Optional["ErrorStatisticsType.CompositeSCP"] = field(
        default=None,
        metadata={
            "name": "CompositeSCP",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    components: Optional["ErrorStatisticsType.Components"] = field(
        default=None,
        metadata={
            "name": "Components",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    unmodeled: Optional["ErrorStatisticsType.Unmodeled"] = field(
        default=None,
        metadata={
            "name": "Unmodeled",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    additional_parms: Optional["ErrorStatisticsType.AdditionalParms"] = field(
        default=None,
        metadata={
            "name": "AdditionalParms",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class CompositeSCP:
        rg: Optional[float] = field(
            default=None,
            metadata={
                "name": "Rg",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        az: Optional[float] = field(
            default=None,
            metadata={
                "name": "Az",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        rg_az: Optional[float] = field(
            default=None,
            metadata={
                "name": "RgAz",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class Components:
        pos_vel_err: Optional["ErrorStatisticsType.Components.PosVelErr"] = field(
            default=None,
            metadata={
                "name": "PosVelErr",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        radar_sensor: Optional["ErrorStatisticsType.Components.RadarSensor"] = field(
            default=None,
            metadata={
                "name": "RadarSensor",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        tropo_error: Optional["ErrorStatisticsType.Components.TropoError"] = field(
            default=None,
            metadata={
                "name": "TropoError",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )
        iono_error: Optional["ErrorStatisticsType.Components.IonoError"] = field(
            default=None,
            metadata={
                "name": "IonoError",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )

        @dataclass
        class PosVelErr:
            frame: Optional[PosVelErrFrame] = field(
                default=None,
                metadata={
                    "name": "Frame",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            p1: Optional[float] = field(
                default=None,
                metadata={
                    "name": "P1",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            p2: Optional[float] = field(
                default=None,
                metadata={
                    "name": "P2",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            p3: Optional[float] = field(
                default=None,
                metadata={
                    "name": "P3",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            v1: Optional[float] = field(
                default=None,
                metadata={
                    "name": "V1",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            v2: Optional[float] = field(
                default=None,
                metadata={
                    "name": "V2",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            v3: Optional[float] = field(
                default=None,
                metadata={
                    "name": "V3",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            corr_coefs: Optional["ErrorStatisticsType.Components.PosVelErr.CorrCoefs"] = field(
                default=None,
                metadata={
                    "name": "CorrCoefs",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            position_decorr: Optional[ErrorDecorrFuncType] = field(
                default=None,
                metadata={
                    "name": "PositionDecorr",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )

            @dataclass
            class CorrCoefs:
                p1_p2: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P1P2",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p1_p3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P1P3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p1_v1: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P1V1",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p1_v2: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P1V2",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p1_v3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P1V3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p2_p3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P2P3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p2_v1: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P2V1",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p2_v2: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P2V2",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p2_v3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P2V3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p3_v1: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P3V1",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p3_v2: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P3V2",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                p3_v3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "P3V3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                v1_v2: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "V1V2",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                v1_v3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "V1V3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                v2_v3: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "V2V3",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )

        @dataclass
        class RadarSensor:
            range_bias: Optional[float] = field(
                default=None,
                metadata={
                    "name": "RangeBias",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            clock_freq_sf: Optional[float] = field(
                default=None,
                metadata={
                    "name": "ClockFreqSF",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            transmit_freq_sf: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TransmitFreqSF",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            range_bias_decorr: Optional[ErrorDecorrFuncType] = field(
                default=None,
                metadata={
                    "name": "RangeBiasDecorr",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )

        @dataclass
        class TropoError:
            tropo_range_vertical: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TropoRangeVertical",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            tropo_range_slant: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TropoRangeSlant",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            tropo_range_decorr: Optional[ErrorDecorrFuncType] = field(
                default=None,
                metadata={
                    "name": "TropoRangeDecorr",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )

        @dataclass
        class IonoError:
            iono_range_vertical: Optional[float] = field(
                default=None,
                metadata={
                    "name": "IonoRangeVertical",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            iono_range_rate_vertical: Optional[float] = field(
                default=None,
                metadata={
                    "name": "IonoRangeRateVertical",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            iono_rg_rg_rate_cc: Optional[float] = field(
                default=None,
                metadata={
                    "name": "IonoRgRgRateCC",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            iono_range_vert_decorr: Optional[ErrorDecorrFuncType] = field(
                default=None,
                metadata={
                    "name": "IonoRangeVertDecorr",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )

    @dataclass
    class Unmodeled:
        xrow: Optional[float] = field(
            default=None,
            metadata={
                "name": "Xrow",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        ycol: Optional[float] = field(
            default=None,
            metadata={
                "name": "Ycol",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        xrow_ycol: Optional[float] = field(
            default=None,
            metadata={
                "name": "XrowYcol",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        unmodeled_decorr: Optional["ErrorStatisticsType.Unmodeled.UnmodeledDecorr"] = field(
            default=None,
            metadata={
                "name": "UnmodeledDecorr",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )

        @dataclass
        class UnmodeledDecorr:
            xrow: Optional[ErrorDecorrFuncType] = field(
                default=None,
                metadata={
                    "name": "Xrow",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            ycol: Optional[ErrorDecorrFuncType] = field(
                default=None,
                metadata={
                    "name": "Ycol",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )

    @dataclass
    class AdditionalParms:
        parameter: List[ParameterType] = field(
            default_factory=list,
            metadata={
                "name": "Parameter",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )


@dataclass
class ImageFormationType:
    rcv_chan_proc: Optional["ImageFormationType.RcvChanProc"] = field(
        default=None,
        metadata={
            "name": "RcvChanProc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    tx_rcv_polarization_proc: Optional[str] = field(
        default=None,
        metadata={
            "name": "TxRcvPolarizationProc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "pattern": r"(([VHXYSE]|RHC|LHC|OTHER[^:]*):([VHXYSE]|RHC|LHC|OTHER[^:]*))|OTHER|UNKNOWN",
        },
    )
    tstart_proc: Optional[float] = field(
        default=None,
        metadata={
            "name": "TStartProc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    tend_proc: Optional[float] = field(
        default=None,
        metadata={
            "name": "TEndProc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    tx_frequency_proc: Optional["ImageFormationType.TxFrequencyProc"] = field(
        default=None,
        metadata={
            "name": "TxFrequencyProc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    segment_identifier: Optional[str] = field(
        default=None,
        metadata={
            "name": "SegmentIdentifier",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    image_form_algo: Optional[ImageFormAlgo] = field(
        default=None,
        metadata={
            "name": "ImageFormAlgo",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    stbeam_comp: Optional[STBeamComp] = field(
        default=None,
        metadata={
            "name": "STBeamComp",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    image_beam_comp: Optional[ImageBeamComp] = field(
        default=None,
        metadata={
            "name": "ImageBeamComp",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    az_autofocus: Optional[AzAutofocus] = field(
        default=None,
        metadata={
            "name": "AzAutofocus",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    rg_autofocus: Optional[RgAutofocus] = field(
        default=None,
        metadata={
            "name": "RgAutofocus",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    processing: List["ImageFormationType.Processing"] = field(
        default_factory=list,
        metadata={
            "name": "Processing",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    polarization_calibration: Optional["ImageFormationType.PolarizationCalibration"] = field(
        default=None,
        metadata={
            "name": "PolarizationCalibration",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class RcvChanProc:
        num_chan_proc: Optional[int] = field(
            default=None,
            metadata={
                "name": "NumChanProc",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        prfscale_factor: Optional[float] = field(
            default=None,
            metadata={
                "name": "PRFScaleFactor",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )
        chan_index: List[int] = field(
            default_factory=list,
            metadata={
                "name": "ChanIndex",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )

    @dataclass
    class TxFrequencyProc:
        min_proc: Optional[float] = field(
            default=None,
            metadata={
                "name": "MinProc",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        max_proc: Optional[float] = field(
            default=None,
            metadata={
                "name": "MaxProc",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class Processing:
        type_value: Optional[str] = field(
            default=None,
            metadata={
                "name": "Type",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        applied: Optional[bool] = field(
            default=None,
            metadata={
                "name": "Applied",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        parameter: List[ParameterType] = field(
            default_factory=list,
            metadata={
                "name": "Parameter",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )

    @dataclass
    class PolarizationCalibration:
        distort_correction_applied: Optional[bool] = field(
            default=None,
            metadata={
                "name": "DistortCorrectionApplied",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        distortion: Optional["ImageFormationType.PolarizationCalibration.Distortion"] = field(
            default=None,
            metadata={
                "name": "Distortion",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

        @dataclass
        class Distortion:
            calibration_date: Optional[XmlDateTime] = field(
                default=None,
                metadata={
                    "name": "CalibrationDate",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            a: Optional[float] = field(
                default=None,
                metadata={
                    "name": "A",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            f1: Optional[ComplexType] = field(
                default=None,
                metadata={
                    "name": "F1",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            q1: Optional[ComplexType] = field(
                default=None,
                metadata={
                    "name": "Q1",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            q2: Optional[ComplexType] = field(
                default=None,
                metadata={
                    "name": "Q2",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            f2: Optional[ComplexType] = field(
                default=None,
                metadata={
                    "name": "F2",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            q3: Optional[ComplexType] = field(
                default=None,
                metadata={
                    "name": "Q3",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            q4: Optional[ComplexType] = field(
                default=None,
                metadata={
                    "name": "Q4",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            gain_error_a: Optional[float] = field(
                default=None,
                metadata={
                    "name": "GainErrorA",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            gain_error_f1: Optional[float] = field(
                default=None,
                metadata={
                    "name": "GainErrorF1",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            gain_error_f2: Optional[float] = field(
                default=None,
                metadata={
                    "name": "GainErrorF2",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            phase_error_f1: Optional[float] = field(
                default=None,
                metadata={
                    "name": "PhaseErrorF1",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            phase_error_f2: Optional[float] = field(
                default=None,
                metadata={
                    "name": "PhaseErrorF2",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )


@dataclass
class LatLonCornerStringType(LatLonType):
    index: Optional[CornerStringType] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class LatLonCornerType(LatLonType):
    index: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
            "min_inclusive": 1,
            "max_inclusive": 4,
        },
    )


@dataclass
class LatLonHAECornerRestrictType(LatLonHAERestrictionType):
    index: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
            "min_inclusive": 1,
            "max_inclusive": 4,
        },
    )


@dataclass
class LatLonHAECornerStringType(LatLonHAEType):
    index: Optional[CornerStringType] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class LineType:
    endpoint: List["LineType.Endpoint"] = field(
        default_factory=list,
        metadata={
            "name": "Endpoint",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "min_occurs": 2,
        },
    )
    size: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )

    @dataclass
    class Endpoint(LatLonType):
        index: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )


@dataclass
class MatchInfoType:
    num_match_types: Optional[int] = field(
        default=None,
        metadata={
            "name": "NumMatchTypes",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    match_type: List["MatchInfoType.MatchType"] = field(
        default_factory=list,
        metadata={
            "name": "MatchType",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "min_occurs": 1,
        },
    )

    @dataclass
    class MatchType:
        type_id: Optional[str] = field(
            default=None,
            metadata={
                "name": "TypeID",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        current_index: Optional[int] = field(
            default=None,
            metadata={
                "name": "CurrentIndex",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )
        num_match_collections: Optional[int] = field(
            default=None,
            metadata={
                "name": "NumMatchCollections",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        match_collection: List["MatchInfoType.MatchType.MatchCollection"] = field(
            default_factory=list,
            metadata={
                "name": "MatchCollection",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )
        index: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )

        @dataclass
        class MatchCollection:
            core_name: Optional[str] = field(
                default=None,
                metadata={
                    "name": "CoreName",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            match_index: Optional[int] = field(
                default=None,
                metadata={
                    "name": "MatchIndex",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            parameter: List[ParameterType] = field(
                default_factory=list,
                metadata={
                    "name": "Parameter",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            index: Optional[int] = field(
                default=None,
                metadata={
                    "type": "Attribute",
                    "required": True,
                },
            )


@dataclass
class Poly1DType:
    coef: List[PolyCoef1DType] = field(
        default_factory=list,
        metadata={
            "name": "Coef",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "min_occurs": 1,
        },
    )
    order1: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class Poly2DType:
    coef: List[PolyCoef2DType] = field(
        default_factory=list,
        metadata={
            "name": "Coef",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "min_occurs": 1,
        },
    )
    order1: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )
    order2: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class PolygonType:
    vertex: List["PolygonType.Vertex"] = field(
        default_factory=list,
        metadata={
            "name": "Vertex",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "min_occurs": 3,
        },
    )
    size: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )

    @dataclass
    class Vertex(LatLonRestrictionType):
        index: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )


@dataclass
class RowColvertexType(RowColType):
    index: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class SCPCOAType:
    scptime: Optional[float] = field(
        default=None,
        metadata={
            "name": "SCPTime",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    arppos: Optional[XYZType] = field(
        default=None,
        metadata={
            "name": "ARPPos",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    arpvel: Optional[XYZType] = field(
        default=None,
        metadata={
            "name": "ARPVel",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    arpacc: Optional[XYZType] = field(
        default=None,
        metadata={
            "name": "ARPAcc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    side_of_track: Optional[SideOfTrack] = field(
        default=None,
        metadata={
            "name": "SideOfTrack",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    slant_range: Optional[float] = field(
        default=None,
        metadata={
            "name": "SlantRange",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    ground_range: Optional[float] = field(
        default=None,
        metadata={
            "name": "GroundRange",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    doppler_cone_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "DopplerConeAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    graze_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "GrazeAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 90.0,
        },
    )
    incidence_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "IncidenceAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 90.0,
        },
    )
    twist_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "TwistAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": -90.0,
            "max_inclusive": 90.0,
        },
    )
    slope_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "SlopeAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 90.0,
        },
    )
    azim_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "AzimAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 360.0,
        },
    )
    layover_ang: Optional[float] = field(
        default=None,
        metadata={
            "name": "LayoverAng",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "min_inclusive": 0.0,
            "max_inclusive": 360.0,
        },
    )


@dataclass
class DirParamType:
    uvect_ecf: Optional[XYZType] = field(
        default=None,
        metadata={
            "name": "UVectECF",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    ss: Optional[float] = field(
        default=None,
        metadata={
            "name": "SS",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    imp_resp_wid: Optional[float] = field(
        default=None,
        metadata={
            "name": "ImpRespWid",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    sgn: Optional[DirParamTypeSgn] = field(
        default=None,
        metadata={
            "name": "Sgn",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    imp_resp_bw: Optional[float] = field(
        default=None,
        metadata={
            "name": "ImpRespBW",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    kctr: Optional[float] = field(
        default=None,
        metadata={
            "name": "KCtr",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    delta_k1: Optional[float] = field(
        default=None,
        metadata={
            "name": "DeltaK1",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    delta_k2: Optional[float] = field(
        default=None,
        metadata={
            "name": "DeltaK2",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    delta_kcoapoly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "DeltaKCOAPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    wgt_type: Optional["DirParamType.WgtType"] = field(
        default=None,
        metadata={
            "name": "WgtType",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    wgt_funct: Optional["DirParamType.WgtFunct"] = field(
        default=None,
        metadata={
            "name": "WgtFunct",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class WgtType:
        window_name: Optional[str] = field(
            default=None,
            metadata={
                "name": "WindowName",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        parameter: List[ParameterType] = field(
            default_factory=list,
            metadata={
                "name": "Parameter",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )

    @dataclass
    class WgtFunct:
        wgt: List[ArrayDouble] = field(
            default_factory=list,
            metadata={
                "name": "Wgt",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 2,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )


@dataclass
class GainPhasePolyType:
    gain_poly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "GainPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    phase_poly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "PhasePoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class GeoInfoType:
    desc: List[ParameterType] = field(
        default_factory=list,
        metadata={
            "name": "Desc",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    point: Optional[LatLonRestrictionType] = field(
        default=None,
        metadata={
            "name": "Point",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    line: Optional[LineType] = field(
        default=None,
        metadata={
            "name": "Line",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    polygon: Optional[PolygonType] = field(
        default=None,
        metadata={
            "name": "Polygon",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    geo_info: List["GeoInfoType"] = field(
        default_factory=list,
        metadata={
            "name": "GeoInfo",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    name: Optional[str] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class ImageDataType:
    pixel_type: Optional[ImagePixelType] = field(
        default=None,
        metadata={
            "name": "PixelType",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    amp_table: Optional["ImageDataType.AmpTable"] = field(
        default=None,
        metadata={
            "name": "AmpTable",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    num_rows: Optional[int] = field(
        default=None,
        metadata={
            "name": "NumRows",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    num_cols: Optional[int] = field(
        default=None,
        metadata={
            "name": "NumCols",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    first_row: Optional[int] = field(
        default=None,
        metadata={
            "name": "FirstRow",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    first_col: Optional[int] = field(
        default=None,
        metadata={
            "name": "FirstCol",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    full_image: Optional["ImageDataType.FullImage"] = field(
        default=None,
        metadata={
            "name": "FullImage",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    scppixel: Optional[RowColType] = field(
        default=None,
        metadata={
            "name": "SCPPixel",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    valid_data: Optional["ImageDataType.ValidData"] = field(
        default=None,
        metadata={
            "name": "ValidData",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class AmpTable:
        amplitude: List[ArrayDouble] = field(
            default_factory=list,
            metadata={
                "name": "Amplitude",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 256,
                "max_occurs": 256,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )

    @dataclass
    class FullImage:
        num_rows: Optional[int] = field(
            default=None,
            metadata={
                "name": "NumRows",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        num_cols: Optional[int] = field(
            default=None,
            metadata={
                "name": "NumCols",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class ValidData:
        vertex: List[RowColvertexType] = field(
            default_factory=list,
            metadata={
                "name": "Vertex",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 3,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )


@dataclass
class PFAType:
    fpn: Optional[XYZType] = field(
        default=None,
        metadata={
            "name": "FPN",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    ipn: Optional[XYZType] = field(
        default=None,
        metadata={
            "name": "IPN",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    polar_ang_ref_time: Optional[float] = field(
        default=None,
        metadata={
            "name": "PolarAngRefTime",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    polar_ang_poly: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "PolarAngPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    spatial_freq_sfpoly: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "SpatialFreqSFPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    krg1: Optional[float] = field(
        default=None,
        metadata={
            "name": "Krg1",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    krg2: Optional[float] = field(
        default=None,
        metadata={
            "name": "Krg2",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    kaz1: Optional[float] = field(
        default=None,
        metadata={
            "name": "Kaz1",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    kaz2: Optional[float] = field(
        default=None,
        metadata={
            "name": "Kaz2",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    stdeskew: Optional["PFAType.STDeskew"] = field(
        default=None,
        metadata={
            "name": "STDeskew",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class STDeskew:
        applied: Optional[bool] = field(
            default=None,
            metadata={
                "name": "Applied",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        stdsphase_poly: Optional[Poly2DType] = field(
            default=None,
            metadata={
                "name": "STDSPhasePoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )


@dataclass
class RMAType:
    rmalgo_type: Optional[RMAlgorithm] = field(
        default=None,
        metadata={
            "name": "RMAlgoType",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    image_type: Optional[RMAImageType] = field(
        default=None,
        metadata={
            "name": "ImageType",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    rmat: Optional["RMAType.RMAT"] = field(
        default=None,
        metadata={
            "name": "RMAT",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    rmcr: Optional["RMAType.RMCR"] = field(
        default=None,
        metadata={
            "name": "RMCR",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    inca: Optional["RMAType.INCA"] = field(
        default=None,
        metadata={
            "name": "INCA",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class RMAT:
        pos_ref: Optional[XYZType] = field(
            default=None,
            metadata={
                "name": "PosRef",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        vel_ref: Optional[XYZType] = field(
            default=None,
            metadata={
                "name": "VelRef",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        dop_cone_ang_ref: Optional[float] = field(
            default=None,
            metadata={
                "name": "DopConeAngRef",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class RMCR:
        pos_ref: Optional[XYZType] = field(
            default=None,
            metadata={
                "name": "PosRef",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        vel_ref: Optional[XYZType] = field(
            default=None,
            metadata={
                "name": "VelRef",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        dop_cone_ang_ref: Optional[float] = field(
            default=None,
            metadata={
                "name": "DopConeAngRef",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class INCA:
        time_capoly: Optional[Poly1DType] = field(
            default=None,
            metadata={
                "name": "TimeCAPoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        r_ca_scp: Optional[float] = field(
            default=None,
            metadata={
                "name": "R_CA_SCP",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        freq_zero: Optional[float] = field(
            default=None,
            metadata={
                "name": "FreqZero",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        drate_sfpoly: Optional[Poly2DType] = field(
            default=None,
            metadata={
                "name": "DRateSFPoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        dop_centroid_poly: Optional[Poly2DType] = field(
            default=None,
            metadata={
                "name": "DopCentroidPoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )
        dop_centroid_coa: Optional[bool] = field(
            default=None,
            metadata={
                "name": "DopCentroidCOA",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )


@dataclass
class RadarCollectionType:
    tx_frequency: Optional["RadarCollectionType.TxFrequency"] = field(
        default=None,
        metadata={
            "name": "TxFrequency",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    ref_freq_index: Optional[int] = field(
        default=None,
        metadata={
            "name": "RefFreqIndex",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    waveform: Optional["RadarCollectionType.Waveform"] = field(
        default=None,
        metadata={
            "name": "Waveform",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    tx_polarization: Optional[str] = field(
        default=None,
        metadata={
            "name": "TxPolarization",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
            "pattern": r"[VHXYSE]|RHC|LHC|UNKNOWN|SEQUENCE|OTHER[^:]*",
        },
    )
    tx_sequence: Optional["RadarCollectionType.TxSequence"] = field(
        default=None,
        metadata={
            "name": "TxSequence",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    rcv_channels: Optional["RadarCollectionType.RcvChannels"] = field(
        default=None,
        metadata={
            "name": "RcvChannels",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    area: Optional["RadarCollectionType.Area"] = field(
        default=None,
        metadata={
            "name": "Area",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    parameter: List[ParameterType] = field(
        default_factory=list,
        metadata={
            "name": "Parameter",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class TxFrequency:
        min: Optional[float] = field(
            default=None,
            metadata={
                "name": "Min",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        max: Optional[float] = field(
            default=None,
            metadata={
                "name": "Max",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class Waveform:
        wfparameters: List["RadarCollectionType.Waveform.WFParameters"] = field(
            default_factory=list,
            metadata={
                "name": "WFParameters",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )

        @dataclass
        class WFParameters:
            tx_pulse_length: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TxPulseLength",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            tx_rfbandwidth: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TxRFBandwidth",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            tx_freq_start: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TxFreqStart",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            tx_fmrate: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TxFMRate",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            rcv_demod_type: Optional[WFParametersRcvDemodType] = field(
                default=None,
                metadata={
                    "name": "RcvDemodType",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            rcv_window_length: Optional[float] = field(
                default=None,
                metadata={
                    "name": "RcvWindowLength",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            adcsample_rate: Optional[float] = field(
                default=None,
                metadata={
                    "name": "ADCSampleRate",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            rcv_ifbandwidth: Optional[float] = field(
                default=None,
                metadata={
                    "name": "RcvIFBandwidth",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            rcv_freq_start: Optional[float] = field(
                default=None,
                metadata={
                    "name": "RcvFreqStart",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            rcv_fmrate: Optional[float] = field(
                default=None,
                metadata={
                    "name": "RcvFMRate",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            index: Optional[int] = field(
                default=None,
                metadata={
                    "type": "Attribute",
                    "required": True,
                },
            )

    @dataclass
    class TxSequence:
        tx_step: List["RadarCollectionType.TxSequence.TxStep"] = field(
            default_factory=list,
            metadata={
                "name": "TxStep",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )

        @dataclass
        class TxStep:
            wfindex: Optional[int] = field(
                default=None,
                metadata={
                    "name": "WFIndex",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            tx_polarization: Optional[str] = field(
                default=None,
                metadata={
                    "name": "TxPolarization",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "pattern": r"[VHXYSE]|RHC|LHC|UNKNOWN|OTHER[^:]*",
                },
            )
            index: Optional[int] = field(
                default=None,
                metadata={
                    "type": "Attribute",
                    "required": True,
                },
            )

    @dataclass
    class RcvChannels:
        chan_parameters: List["RadarCollectionType.RcvChannels.ChanParameters"] = field(
            default_factory=list,
            metadata={
                "name": "ChanParameters",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )

        @dataclass
        class ChanParameters:
            tx_rcv_polarization: Optional[str] = field(
                default=None,
                metadata={
                    "name": "TxRcvPolarization",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                    "pattern": r"(([VHXYSE]|RHC|LHC|OTHER[^:]*):([VHXYSE]|RHC|LHC|OTHER[^:]*))|OTHER|UNKNOWN",
                },
            )
            rcv_apcindex: Optional[int] = field(
                default=None,
                metadata={
                    "name": "RcvAPCIndex",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            index: Optional[int] = field(
                default=None,
                metadata={
                    "type": "Attribute",
                    "required": True,
                },
            )

    @dataclass
    class Area:
        corner: Optional["RadarCollectionType.Area.Corner"] = field(
            default=None,
            metadata={
                "name": "Corner",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        plane: Optional["RadarCollectionType.Area.Plane"] = field(
            default=None,
            metadata={
                "name": "Plane",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
            },
        )

        @dataclass
        class Corner:
            acp: List[LatLonHAECornerRestrictType] = field(
                default_factory=list,
                metadata={
                    "name": "ACP",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "min_occurs": 4,
                    "max_occurs": 4,
                },
            )

        @dataclass
        class Plane:
            ref_pt: Optional["RadarCollectionType.Area.Plane.RefPt"] = field(
                default=None,
                metadata={
                    "name": "RefPt",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            xdir: Optional["RadarCollectionType.Area.Plane.XDir"] = field(
                default=None,
                metadata={
                    "name": "XDir",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            ydir: Optional["RadarCollectionType.Area.Plane.YDir"] = field(
                default=None,
                metadata={
                    "name": "YDir",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            segment_list: Optional["RadarCollectionType.Area.Plane.SegmentList"] = field(
                default=None,
                metadata={
                    "name": "SegmentList",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )
            orientation: Optional[OrientationType] = field(
                default=None,
                metadata={
                    "name": "Orientation",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                },
            )

            @dataclass
            class RefPt:
                ecf: Optional[XYZType] = field(
                    default=None,
                    metadata={
                        "name": "ECF",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                line: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "Line",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                sample: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "Sample",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                name: Optional[str] = field(
                    default=None,
                    metadata={
                        "type": "Attribute",
                    },
                )

            @dataclass
            class XDir:
                uvect_ecf: Optional[XYZType] = field(
                    default=None,
                    metadata={
                        "name": "UVectECF",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                line_spacing: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "LineSpacing",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                num_lines: Optional[int] = field(
                    default=None,
                    metadata={
                        "name": "NumLines",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                first_line: Optional[int] = field(
                    default=None,
                    metadata={
                        "name": "FirstLine",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )

            @dataclass
            class YDir:
                uvect_ecf: Optional[XYZType] = field(
                    default=None,
                    metadata={
                        "name": "UVectECF",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                sample_spacing: Optional[float] = field(
                    default=None,
                    metadata={
                        "name": "SampleSpacing",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                num_samples: Optional[int] = field(
                    default=None,
                    metadata={
                        "name": "NumSamples",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )
                first_sample: Optional[int] = field(
                    default=None,
                    metadata={
                        "name": "FirstSample",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "required": True,
                    },
                )

            @dataclass
            class SegmentList:
                segment: List["RadarCollectionType.Area.Plane.SegmentList.Segment"] = field(
                    default_factory=list,
                    metadata={
                        "name": "Segment",
                        "type": "Element",
                        "namespace": "urn:SICD:1.3.0",
                        "min_occurs": 1,
                    },
                )
                size: Optional[int] = field(
                    default=None,
                    metadata={
                        "type": "Attribute",
                        "required": True,
                    },
                )

                @dataclass
                class Segment:
                    start_line: Optional[int] = field(
                        default=None,
                        metadata={
                            "name": "StartLine",
                            "type": "Element",
                            "namespace": "urn:SICD:1.3.0",
                            "required": True,
                        },
                    )
                    start_sample: Optional[int] = field(
                        default=None,
                        metadata={
                            "name": "StartSample",
                            "type": "Element",
                            "namespace": "urn:SICD:1.3.0",
                            "required": True,
                        },
                    )
                    end_line: Optional[int] = field(
                        default=None,
                        metadata={
                            "name": "EndLine",
                            "type": "Element",
                            "namespace": "urn:SICD:1.3.0",
                            "required": True,
                        },
                    )
                    end_sample: Optional[int] = field(
                        default=None,
                        metadata={
                            "name": "EndSample",
                            "type": "Element",
                            "namespace": "urn:SICD:1.3.0",
                            "required": True,
                        },
                    )
                    identifier: Optional[str] = field(
                        default=None,
                        metadata={
                            "name": "Identifier",
                            "type": "Element",
                            "namespace": "urn:SICD:1.3.0",
                            "required": True,
                        },
                    )
                    index: Optional[int] = field(
                        default=None,
                        metadata={
                            "type": "Attribute",
                            "required": True,
                        },
                    )


@dataclass
class RadiometricType:
    noise_level: Optional["RadiometricType.NoiseLevel"] = field(
        default=None,
        metadata={
            "name": "NoiseLevel",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    rcssfpoly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "RCSSFPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    sigma_zero_sfpoly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "SigmaZeroSFPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    beta_zero_sfpoly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "BetaZeroSFPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    gamma_zero_sfpoly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "GammaZeroSFPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class NoiseLevel:
        noise_level_type: Optional[NoiseLevelType] = field(
            default=None,
            metadata={
                "name": "NoiseLevelType",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        noise_poly: Optional[Poly2DType] = field(
            default=None,
            metadata={
                "name": "NoisePoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )


@dataclass
class RgAzCompType:
    az_sf: Optional[float] = field(
        default=None,
        metadata={
            "name": "AzSF",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    kaz_poly: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "KazPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class TimelineType:
    collect_start: Optional[XmlDateTime] = field(
        default=None,
        metadata={
            "name": "CollectStart",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    collect_duration: Optional[float] = field(
        default=None,
        metadata={
            "name": "CollectDuration",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    ipp: Optional["TimelineType.IPP"] = field(
        default=None,
        metadata={
            "name": "IPP",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class IPP:
        set: List["TimelineType.IPP.Set"] = field(
            default_factory=list,
            metadata={
                "name": "Set",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )

        @dataclass
        class Set:
            tstart: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TStart",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            tend: Optional[float] = field(
                default=None,
                metadata={
                    "name": "TEnd",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            ippstart: Optional[int] = field(
                default=None,
                metadata={
                    "name": "IPPStart",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            ippend: Optional[int] = field(
                default=None,
                metadata={
                    "name": "IPPEnd",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            ipppoly: Optional[Poly1DType] = field(
                default=None,
                metadata={
                    "name": "IPPPoly",
                    "type": "Element",
                    "namespace": "urn:SICD:1.3.0",
                    "required": True,
                },
            )
            index: Optional[int] = field(
                default=None,
                metadata={
                    "type": "Attribute",
                    "required": True,
                },
            )


@dataclass
class XYZPolyType:
    x: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "X",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    y: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "Y",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    z: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "Z",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class AntParamType:
    xaxis_poly: Optional[XYZPolyType] = field(
        default=None,
        metadata={
            "name": "XAxisPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    yaxis_poly: Optional[XYZPolyType] = field(
        default=None,
        metadata={
            "name": "YAxisPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    freq_zero: Optional[float] = field(
        default=None,
        metadata={
            "name": "FreqZero",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    eb: Optional["AntParamType.EB"] = field(
        default=None,
        metadata={
            "name": "EB",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    array: Optional[GainPhasePolyType] = field(
        default=None,
        metadata={
            "name": "Array",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    elem: Optional[GainPhasePolyType] = field(
        default=None,
        metadata={
            "name": "Elem",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    gain_bspoly: Optional[Poly1DType] = field(
        default=None,
        metadata={
            "name": "GainBSPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    ebfreq_shift: Optional[bool] = field(
        default=None,
        metadata={
            "name": "EBFreqShift",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    mlfreq_dilation: Optional[bool] = field(
        default=None,
        metadata={
            "name": "MLFreqDilation",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class EB:
        dcxpoly: Optional[Poly1DType] = field(
            default=None,
            metadata={
                "name": "DCXPoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        dcypoly: Optional[Poly1DType] = field(
            default=None,
            metadata={
                "name": "DCYPoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )


@dataclass
class GeoDataType:
    earth_model: Optional[EarthModel] = field(
        default=None,
        metadata={
            "name": "EarthModel",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    scp: Optional["GeoDataType.SCP"] = field(
        default=None,
        metadata={
            "name": "SCP",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    image_corners: Optional["GeoDataType.ImageCorners"] = field(
        default=None,
        metadata={
            "name": "ImageCorners",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    valid_data: Optional[PolygonType] = field(
        default=None,
        metadata={
            "name": "ValidData",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    geo_info: List[GeoInfoType] = field(
        default_factory=list,
        metadata={
            "name": "GeoInfo",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class SCP:
        ecf: Optional[XYZType] = field(
            default=None,
            metadata={
                "name": "ECF",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )
        llh: Optional[LatLonHAERestrictionType] = field(
            default=None,
            metadata={
                "name": "LLH",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "required": True,
            },
        )

    @dataclass
    class ImageCorners:
        icp: List[LatLonCornerStringType] = field(
            default_factory=list,
            metadata={
                "name": "ICP",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 4,
                "max_occurs": 4,
            },
        )


@dataclass
class GridType:
    image_plane: Optional[ImagePlane] = field(
        default=None,
        metadata={
            "name": "ImagePlane",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    type_value: Optional[ImageGridType] = field(
        default=None,
        metadata={
            "name": "Type",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    time_coapoly: Optional[Poly2DType] = field(
        default=None,
        metadata={
            "name": "TimeCOAPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    row: Optional[DirParamType] = field(
        default=None,
        metadata={
            "name": "Row",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    col: Optional[DirParamType] = field(
        default=None,
        metadata={
            "name": "Col",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )


@dataclass
class XYZPolyAttributeType(XYZPolyType):
    index: Optional[int] = field(
        default=None,
        metadata={
            "type": "Attribute",
            "required": True,
        },
    )


@dataclass
class AntennaType:
    tx: Optional[AntParamType] = field(
        default=None,
        metadata={
            "name": "Tx",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    rcv: Optional[AntParamType] = field(
        default=None,
        metadata={
            "name": "Rcv",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    two_way: Optional[AntParamType] = field(
        default=None,
        metadata={
            "name": "TwoWay",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )


@dataclass
class PositionType:
    arppoly: Optional[XYZPolyType] = field(
        default=None,
        metadata={
            "name": "ARPPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
            "required": True,
        },
    )
    grppoly: Optional[XYZPolyType] = field(
        default=None,
        metadata={
            "name": "GRPPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    tx_apcpoly: Optional[XYZPolyType] = field(
        default=None,
        metadata={
            "name": "TxAPCPoly",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )
    rcv_apc: Optional["PositionType.RcvAPC"] = field(
        default=None,
        metadata={
            "name": "RcvAPC",
            "type": "Element",
            "namespace": "urn:SICD:1.3.0",
        },
    )

    @dataclass
    class RcvAPC:
        rcv_apcpoly: List[XYZPolyAttributeType] = field(
            default_factory=list,
            metadata={
                "name": "RcvAPCPoly",
                "type": "Element",
                "namespace": "urn:SICD:1.3.0",
                "min_occurs": 1,
            },
        )
        size: Optional[int] = field(
            default=None,
            metadata={
                "type": "Attribute",
                "required": True,
            },
        )


@dataclass
class SICD:
    class Meta:
        namespace = "urn:SICD:1.3.0"

    collection_info: Optional[CollectionInfoType] = field(
        default=None,
        metadata={
            "name": "CollectionInfo",
            "type": "Element",
            "required": True,
        },
    )
    image_creation: Optional[ImageCreationType] = field(
        default=None,
        metadata={
            "name": "ImageCreation",
            "type": "Element",
        },
    )
    image_data: Optional[ImageDataType] = field(
        default=None,
        metadata={
            "name": "ImageData",
            "type": "Element",
            "required": True,
        },
    )
    geo_data: Optional[GeoDataType] = field(
        default=None,
        metadata={
            "name": "GeoData",
            "type": "Element",
            "required": True,
        },
    )
    grid: Optional[GridType] = field(
        default=None,
        metadata={
            "name": "Grid",
            "type": "Element",
            "required": True,
        },
    )
    timeline: Optional[TimelineType] = field(
        default=None,
        metadata={
            "name": "Timeline",
            "type": "Element",
            "required": True,
        },
    )
    position: Optional[PositionType] = field(
        default=None,
        metadata={
            "name": "Position",
            "type": "Element",
            "required": True,
        },
    )
    radar_collection: Optional[RadarCollectionType] = field(
        default=None,
        metadata={
            "name": "RadarCollection",
            "type": "Element",
            "required": True,
        },
    )
    image_formation: Optional[ImageFormationType] = field(
        default=None,
        metadata={
            "name": "ImageFormation",
            "type": "Element",
            "required": True,
        },
    )
    scpcoa: Optional[SCPCOAType] = field(
        default=None,
        metadata={
            "name": "SCPCOA",
            "type": "Element",
            "required": True,
        },
    )
    radiometric: Optional[RadiometricType] = field(
        default=None,
        metadata={
            "name": "Radiometric",
            "type": "Element",
        },
    )
    antenna: Optional[AntennaType] = field(
        default=None,
        metadata={
            "name": "Antenna",
            "type": "Element",
        },
    )
    error_statistics: Optional[ErrorStatisticsType] = field(
        default=None,
        metadata={
            "name": "ErrorStatistics",
            "type": "Element",
        },
    )
    match_info: Optional[MatchInfoType] = field(
        default=None,
        metadata={
            "name": "MatchInfo",
            "type": "Element",
        },
    )
    rg_az_comp: Optional[RgAzCompType] = field(
        default=None,
        metadata={
            "name": "RgAzComp",
            "type": "Element",
        },
    )
    pfa: Optional[PFAType] = field(
        default=None,
        metadata={
            "name": "PFA",
            "type": "Element",
        },
    )
    rma: Optional[RMAType] = field(
        default=None,
        metadata={
            "name": "RMA",
            "type": "Element",
        },
    )



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

from math import degrees, radians
from typing import Any, Dict, List, Optional

import numpy as np
import pyproj
from pyproj.enums import TransformDirection

from .coordinates import LLA_PROJ, GeodeticWorldCoordinate, ImageCoordinate
from .elevation_model import ElevationModel
from .sensor_model import SensorModel


class GDALAffineSensorModel(SensorModel):
    """
    GDAL provides a simple affine transform used to convert XY pixel values to longitude,
    latitude. See https://gdal.org/tutorials/geotransforms_tut.html

    transform[0] x-coordinate of the upper-left corner of the upper-left pixel.
    transform[1] w-e pixel resolution / pixel width.
    transform[2] row rotation (typically zero).
    transform[3] y-coordinate of the upper-left corner of the upper-left pixel.
    transform[4] column rotation (typically zero).
    transform[5] n-s pixel resolution / pixel height (negative value for a north-up image).

    The necessary transform matrix can be obtained from a dataset using the GetGeoTransform() operation.
    """

    def __init__(self, geo_transform: List, proj_wkt: Optional[str] = None) -> None:
        """
        Construct the sensor model from the affine transform provided by transform

        :param geo_transform: the 6 coefficients of the affine transform
        :param proj_wkt: the well known text string of the CRS used by the image

        :return: None
        """
        super().__init__()
        try:
            # Put the geo transform parameters into a matrix form that makes the image to world calculation
            # matrix multiplication. In this arrangement the 2D image coordinates should be expanded to have a
            # 1.0 at the end to capture the constant. The third row is being added, so we can invert this matrix
            # to obtain the corresponding world to image coefficients.
            self.transform = np.array(
                [
                    [geo_transform[1], geo_transform[2], geo_transform[0]],
                    [geo_transform[4], geo_transform[5], geo_transform[3]],
                    [0.0, 0.0, 1.0],
                ]
            )
            # Use NumPy to calculate an inverse transform
            self.inv_transform = np.linalg.inv(self.transform)

            self.image_to_wgs84 = None
            if proj_wkt:
                self.image_to_wgs84 = pyproj.Transformer.from_crs(
                    pyproj.CRS.from_string(proj_wkt), LLA_PROJ.crs, always_xy=True
                )

        except np.linalg.LinAlgError:
            raise ValueError("GeoTransform can not be inverted. Not a valid matrix for a sensor model.")

    def image_to_world(
        self,
        image_coordinate: ImageCoordinate,
        elevation_model: Optional[ElevationModel] = None,
        options: Optional[Dict[str, Any]] = None,
    ) -> GeodeticWorldCoordinate:
        """
        This function returns the longitude, latitude, elevation world coordinate associated with the x, y coordinate
        of any pixel in the image. The GDAL Geo Transforms do not provide any information about elevation, so it will
        always be 0.0 unless the optional elevation model is provided.

        :param image_coordinate: the x, y image coordinate
        :param elevation_model: an optional elevation model used to transform the coordinate
        :param options: an optional dictionary of hints, does not support any hints
        :return: the longitude, latitude, elevation world coordinate
        """
        # The transform is expecting coordinates [x, y, 1.0] as an input.
        augmented_image_coord = np.append(image_coordinate.coordinate, [1.0])
        image_crs_coordinate = np.matmul(self.transform, augmented_image_coord)
        if self.image_to_wgs84 is not None:
            lonlat_coordinate = self.image_to_wgs84.transform(
                image_crs_coordinate[0],
                image_crs_coordinate[1],
                image_crs_coordinate[2],
                radians=False,
                direction=TransformDirection.FORWARD,
            )
        else:
            lonlat_coordinate = image_crs_coordinate
        world_coordinate = GeodeticWorldCoordinate([radians(lonlat_coordinate[0]), radians(lonlat_coordinate[1]), 0.0])
        if elevation_model:
            elevation_model.set_elevation(world_coordinate)

        return world_coordinate

    def world_to_image(self, world_coordinate: GeodeticWorldCoordinate) -> ImageCoordinate:
        """
        This function returns the x, y image coordinate associated with a given longitude, latitude, elevation world
        coordinate.

        :param world_coordinate: the longitude, latitude, elevation world coordinate

        :return: the x, y image coordinate
        """
        # The GDAL geo transform does not support elevation data. The inverse transform was created assuming the input
        # coordinate is a 2D geo + 1.0 (i.e. [longitude, latitude, 1.0]
        if self.image_to_wgs84 is not None:
            image_crs_coordinate = np.array(
                self.image_to_wgs84.transform(
                    degrees(world_coordinate.longitude),
                    degrees(world_coordinate.latitude),
                    1.0,
                    radians=False,
                    direction=TransformDirection.INVERSE,
                )
            )
        else:
            image_crs_coordinate = np.array((degrees(world_coordinate.longitude), degrees(world_coordinate.latitude), 1.0))
        xy_coordinate = np.matmul(self.inv_transform, image_crs_coordinate)
        return ImageCoordinate([xy_coordinate[0], xy_coordinate[1]])



#  Copyright 2024 Amazon.com, Inc. or its affiliates.

from abc import ABC, abstractmethod
from collections import namedtuple
from dataclasses import dataclass

from aws.osml.photogrammetry import GeodeticWorldCoordinate

MapTileId = namedtuple("TildId", "tile_matrix tile_row tile_col")
MapTileId.__doc__ = """
This type represents the unique id of a map tile within a map tile set. It is implemented as a named tuple to make
use of that constructs immutability and hashing features.
"""

MapTileSize = namedtuple("MapTileSize", "width height")
MapTileSize.__doc__ = """
This type represents the size of a map tile (width, height). These dimensions are typically the same (i.e. square
tiles) but this is not required.
"""

MapTileBounds = namedtuple("MapTileBounds", "min_lon min_lat max_lon max_lat")
MapTileBounds.__doc__ = """
This type represents the geodetic bounds of a map tile (min_lon, min_lat, max_lon, max_lat).
"""


@dataclass
class MapTile:
    """
    This dataclass provides a description of a map tile that is part of a well known tile set.
    """

    id: MapTileId
    size: MapTileSize
    bounds: MapTileBounds


class MapTileSet(ABC):
    """
    This class provides an abstract interface to a well known set of map tiles.
    """

    @property
    @abstractmethod
    def tile_matrix_set_id(self) -> str:
        """
        Get the identifier for this map tile set. This is the tile matrix set ID in the OGC definitions.

        :return: the tile matrix set ID
        """

    @abstractmethod
    def get_tile(self, tile_id: MapTileId) -> MapTile:
        """
        Get a description of the tile identified by a specific map tile ID.

        :param tile_id: the tile ID
        :return: the tile description
        """

    @abstractmethod
    def get_tile_for_location(self, world_coordinate: GeodeticWorldCoordinate, tile_matrix: int) -> MapTile:
        """
        Get a description of the tile containing a specific world location.

        :param world_coordinate: the location in the world
        :param tile_matrix: the tile_matrix or zoom level of interest
        :return: the tile description
        """

    def get_tile_matrix_limits_for_area(
        self, boundary_coordinates: list[GeodeticWorldCoordinate], tile_matrix: int
    ) -> tuple[int, int, int, int]:
        """
        Get a list of all tiles that intersect a specific area.

        :param boundary_coordinates: the boundary of the area
        :param tile_matrix: the tile_matrix or zoom level of interest
        :return: the (min_col, min_row, max_col, max_row) limits of tiles containing all points
        """
        map_tiles_for_corners = [
            self.get_tile_for_location(world_corner, tile_matrix=tile_matrix) for world_corner in boundary_coordinates
        ]
        tile_rows = [tile_id.id.tile_row for tile_id in map_tiles_for_corners]
        tile_cols = [tile_id.id.tile_col for tile_id in map_tiles_for_corners]

        return min(tile_cols), min(tile_rows), max(tile_cols), max(tile_rows)



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

import logging
import re
from typing import Dict, List, Optional, Tuple

from defusedxml import ElementTree
from osgeo import gdal, gdalconst

from aws.osml.photogrammetry import SensorModel

from .dynamic_range_adjustment import DRAParameters
from .sensor_model_factory import SensorModelFactory, SensorModelTypes
from .typing import RangeAdjustmentType

logger = logging.getLogger(__name__)


def load_gdal_dataset(image_path: str) -> Tuple[gdal.Dataset, Optional[SensorModel]]:
    """
    This function loads a GDAL raster dataset from the path provided and constructs a camera model
    abstraction used to georeference locations on this image.

    :param image_path: the path to the raster data, may be a local path or a virtual file system
        (e.g. /vsis3/...)
    :return: the raster dataset and sensor model
    """
    try:
        logger.info("GDAL attempted to load image: %s", image_path)
        ds = gdal.Open(image_path)
        if ds is None:
            raise RuntimeError("GDAL Unable to load dataset and UseExceptions is not enabled.")
    except RuntimeError:
        logger.info("Skipping: %s - GDAL Unable to Process", image_path)
        raise ValueError("GDAL Unable to Load: {}".format(image_path))

    # Get a GDAL Geo Transform and any available GCPs
    geo_transform = ds.GetGeoTransform(can_return_null=True)
    ground_control_points = ds.GetGCPs()
    proj_wkt = ds.GetProjection()

    # If this image has NITF TREs defined parse them
    parsed_tres = None
    xml_tres = ds.GetMetadata("xml:TRE")
    if xml_tres is not None and len(xml_tres) > 0:
        parsed_tres = ElementTree.fromstring(xml_tres[0])

    # If this image has NITF DES segments read them
    xml_dess = ds.GetMetadata("xml:DES")

    selected_sensor_model_types = [
        SensorModelTypes.AFFINE,
        SensorModelTypes.PROJECTIVE,
        SensorModelTypes.RPC,
        SensorModelTypes.SICD,
        SensorModelTypes.RSM,
    ]
    # Create the best sensor model available
    sensor_model = SensorModelFactory(
        ds.RasterXSize,
        ds.RasterYSize,
        xml_tres=parsed_tres,
        xml_dess=xml_dess,
        geo_transform=geo_transform,
        proj_wkt=proj_wkt,
        ground_control_points=ground_control_points,
        selected_sensor_model_types=selected_sensor_model_types,
    ).build()

    return ds, sensor_model


def get_minmax_for_type(gdal_type: int) -> Tuple[float, float]:
    """
    This function computes the minimum and maximum values that can be stored in a given GDAL pixel type.

    :param gdal_type: the pixel type
    :return: tuple of min, max values
    """
    min_value = 0
    max_value = 255
    if gdal_type == gdalconst.GDT_Byte:
        min_value = 0
        max_value = 2**8 - 1
    elif gdal_type == gdalconst.GDT_UInt16:
        min_value = 0
        max_value = 2**16 - 1
    elif gdal_type == gdalconst.GDT_Int16:
        min_value = -(2**15)
        max_value = 2**15 - 1
    elif gdal_type == gdalconst.GDT_UInt32:
        min_value = 0
        max_value = 2**32 - 1
    elif gdal_type == gdalconst.GDT_Int32:
        min_value = -(2**31)
        max_value = 2**31 - 1
    elif gdal_type == gdalconst.GDT_UInt64:
        min_value = 0
        max_value = 2**64 - 1
    elif gdal_type == gdalconst.GDT_Int64:
        min_value = -(2**63)
        max_value = 2**63 - 1
    elif gdal_type == gdalconst.GDT_Float32:
        min_value = -3.4028235e38
        max_value = 3.4028235e38
    elif gdal_type == gdalconst.GDT_Float64:
        min_value = -1.7976931348623157e308
        max_value = 1.7976931348623157e308
    else:
        logger.warning("Image uses unsupported GDAL datatype {}. Defaulting to [0,255] range".format(gdal_type))

    return min_value, max_value


def get_type_and_scales(
    raster_dataset: gdal.Dataset,
    desired_output_type: Optional[int] = None,
    range_adjustment: RangeAdjustmentType = RangeAdjustmentType.NONE,
) -> Tuple[int, List[List[int]]]:
    """
    Get type and scales of a provided raster dataset

    :param raster_dataset: the raster dataset containing the region
    :param desired_output_type: type to be output after dynamic range adjustments
    :param range_adjustment: the type of pixel scaling effort to

    :return: a tuple containing type and scales
    """
    scale_params = []
    output_type = gdalconst.GDT_Byte
    num_bands = raster_dataset.RasterCount
    for band_num in range(1, num_bands + 1):
        band = raster_dataset.GetRasterBand(band_num)
        band_type = band.DataType
        min_value, max_value = get_minmax_for_type(band_type)

        if desired_output_type is None:
            output_type = band_type
            output_min = min_value
            output_max = max_value
        else:
            output_type = desired_output_type
            output_min, output_max = get_minmax_for_type(desired_output_type)

        # If a range adjustment is requested compute the range of source pixel values that will be mapped to the full
        # output range.
        selected_min = min_value
        selected_max = max_value
        if range_adjustment is not RangeAdjustmentType.NONE:
            # GetStatistics(1,1) means it is ok to approximate but force computation is stats not already available
            stats = band.GetStatistics(1, 1)
            min_value = stats[0]
            max_value = stats[1]

            num_buckets = int(max_value - min_value)
            if band_type == gdalconst.GDT_Float32 or band_type == gdalconst.GDT_Float64:
                num_buckets = 255

            dra = DRAParameters.from_counts(
                counts=band.GetHistogram(
                    buckets=num_buckets, max=max_value, min=min_value, include_out_of_range=1, approx_ok=1
                ),
                first_bucket_value=min_value,
                last_bucket_value=max_value,
            )

            if range_adjustment == RangeAdjustmentType.DRA:
                selected_min = dra.suggested_min_value
                selected_max = dra.suggested_max_value
            elif range_adjustment == RangeAdjustmentType.MINMAX:
                selected_min = dra.actual_min_value
                selected_max = dra.actual_max_value
            else:
                logger.warning(f"Unknown range adjustment selected {range_adjustment}. Skipping.")

        band_scale_parameters = [selected_min, selected_max, output_min, output_max]
        scale_params.append(band_scale_parameters)

    return output_type, scale_params


def get_image_extension(image_path: str) -> str:
    """
    Get the image extension based on the provided image path

    :param image_path: an image path

    :return: image extension
    """
    possible_extensions = get_extensions_from_driver(image_path)
    selected_extension = select_extension(image_path, possible_extensions)
    image_extension = normalize_extension(selected_extension)
    logger.info("Image extension: {}".format(image_extension))
    return image_extension


def select_extension(image_path: str, possible_extensions: List[str]) -> str:
    """
    Check to see if provided image path contains a known possible extensions

    :param image_path: an image path
    :param possible_extensions: list of possible extensions

    :return: select extension
    """
    selected_extension = "UNKNOWN"
    for i, possible_extension in enumerate(possible_extensions):
        if i == 0:
            selected_extension = possible_extension.upper()
        elif f".{possible_extension}".upper() in image_path.upper():
            selected_extension = possible_extension.upper()
    return selected_extension


def normalize_extension(unnormalized_extension: str) -> str:
    """
    Convert the extension into a proper formatted string

    :param unnormalized_extension: an unnormalized extension

    :return: normalized extension
    """
    normalized_extension = unnormalized_extension.upper()
    if re.search(r"ni?tf", normalized_extension, re.IGNORECASE):
        normalized_extension = "NITF"
    elif re.search(r"tif{1,2}", normalized_extension, re.IGNORECASE):
        normalized_extension = "TIFF"
    elif re.search(r"jpe?g", normalized_extension, re.IGNORECASE):
        normalized_extension = "JPEG"
    return normalized_extension


def get_extensions_from_driver(image_path: str) -> List[str]:
    """
    Returns a list of driver extensions

    :param image_path: an image path

    :return: driver extensions
    """
    driver_extension_lookup = get_gdal_driver_extensions()
    info = gdal.Info(image_path, format="json")
    driver_long_name = info.get("driverLongName")
    return driver_extension_lookup.get(driver_long_name, [])


def get_gdal_driver_extensions() -> Dict[str, List]:
    """
    Returns a list of gdal driver extensions

    :return: gdal driver extensions
    """
    driver_lookup = {}
    for i in range(gdal.GetDriverCount()):
        drv = gdal.GetDriver(i)
        driver_name = drv.GetMetadataItem(gdal.DMD_LONGNAME)
        driver_extensions = drv.GetMetadataItem(gdal.DMD_EXTENSIONS)
        if driver_extensions:
            extension_list = driver_extensions.strip().split(" ")
            driver_lookup[driver_name] = extension_list
    return driver_lookup



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from dataclasses import dataclass, field
from typing import List, Optional

__NAMESPACE__ = "urn:SFA:1.2.0"


@dataclass
class GeometryType:
    pass


@dataclass
class ParameterType:
    parameter_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "ParameterName",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    value: Optional[float] = field(
        default=None,
        metadata={
            "name": "Value",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class PrimeMeridianType:
    name: Optional[str] = field(
        default=None,
        metadata={
            "name": "Name",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    longitude: Optional[float] = field(
        default=None,
        metadata={
            "name": "Longitude",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class ProjectionType:
    projection_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "ProjectionName",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class SpheriodType:
    spheriod_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "SpheriodName",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    semi_major_axis: Optional[float] = field(
        default=None,
        metadata={
            "name": "SemiMajorAxis",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    inverse_flattening: Optional[float] = field(
        default=None,
        metadata={
            "name": "InverseFlattening",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class UNITType:
    unit_name: Optional[str] = field(
        default=None,
        metadata={
            "name": "UnitName",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    conversion_factor: Optional[float] = field(
        default=None,
        metadata={
            "name": "ConversionFactor",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class CurveType(GeometryType):
    pass


@dataclass
class DatumType:
    spheroid: Optional[SpheriodType] = field(
        default=None,
        metadata={
            "name": "Spheroid",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class GeometryCollectionType(GeometryType):
    pass


@dataclass
class PointType(GeometryType):
    x: Optional[float] = field(
        default=None,
        metadata={
            "name": "X",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    y: Optional[float] = field(
        default=None,
        metadata={
            "name": "Y",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    z: Optional[float] = field(
        default=None,
        metadata={
            "name": "Z",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )
    m: Optional[float] = field(
        default=None,
        metadata={
            "name": "M",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )


@dataclass
class SurfaceType(GeometryType):
    pass


@dataclass
class GeocentricCoordinateSystemType:
    csname: Optional[str] = field(
        default=None,
        metadata={
            "name": "Csname",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    datum: Optional[DatumType] = field(
        default=None,
        metadata={
            "name": "Datum",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    prime_meridian: Optional[PrimeMeridianType] = field(
        default=None,
        metadata={
            "name": "PrimeMeridian",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    linear_unit: Optional[str] = field(
        default=None,
        metadata={
            "name": "LinearUnit",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class GeographicCoordinateSystemType:
    csname: Optional[str] = field(
        default=None,
        metadata={
            "name": "Csname",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    datum: Optional[DatumType] = field(
        default=None,
        metadata={
            "name": "Datum",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    prime_meridian: Optional[PrimeMeridianType] = field(
        default=None,
        metadata={
            "name": "PrimeMeridian",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    angular_unit: Optional[str] = field(
        default=None,
        metadata={
            "name": "AngularUnit",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    linear_unit: Optional[str] = field(
        default=None,
        metadata={
            "name": "LinearUnit",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )


@dataclass
class LineStringType(CurveType):
    vertex: List[PointType] = field(
        default_factory=list,
        metadata={
            "name": "Vertex",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "min_occurs": 2,
        },
    )


@dataclass
class MultiCurveType(GeometryCollectionType):
    pass


@dataclass
class MultiPointType(GeometryCollectionType):
    vertex: List[PointType] = field(
        default_factory=list,
        metadata={
            "name": "Vertex",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "min_occurs": 2,
        },
    )


@dataclass
class MultiSurfaceType(GeometryCollectionType):
    pass


@dataclass
class LineType(LineStringType):
    pass


@dataclass
class LinearRingType(LineStringType):
    pass


@dataclass
class ProjectedCoordinateSystemType:
    csname: Optional[str] = field(
        default=None,
        metadata={
            "name": "Csname",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    geographic_coordinate_system: Optional[GeographicCoordinateSystemType] = field(
        default=None,
        metadata={
            "name": "GeographicCoordinateSystem",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    projection: Optional[ProjectionType] = field(
        default=None,
        metadata={
            "name": "Projection",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )
    parameter: Optional[ParameterType] = field(
        default=None,
        metadata={
            "name": "Parameter",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )
    linear_unit: Optional[str] = field(
        default=None,
        metadata={
            "name": "LinearUnit",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "required": True,
        },
    )


@dataclass
class AbstractReferenceSystemType:
    projected_coordinate_system: Optional[ProjectedCoordinateSystemType] = field(
        default=None,
        metadata={
            "name": "ProjectedCoordinateSystem",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )
    geographic_coordinate_system: Optional[GeographicCoordinateSystemType] = field(
        default=None,
        metadata={
            "name": "GeographicCoordinateSystem",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )
    geocentric_coordinate_system: Optional[GeocentricCoordinateSystemType] = field(
        default=None,
        metadata={
            "name": "GeocentricCoordinateSystem",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )


@dataclass
class MultiLineStringType(MultiCurveType):
    element: List[LineType] = field(
        default_factory=list,
        metadata={
            "name": "Element",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )


@dataclass
class PolygonType(SurfaceType):
    ring: List[LinearRingType] = field(
        default_factory=list,
        metadata={
            "name": "Ring",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "min_occurs": 1,
        },
    )


@dataclass
class MultiPolygonType(MultiSurfaceType):
    element: List[PolygonType] = field(
        default_factory=list,
        metadata={
            "name": "Element",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
        },
    )


@dataclass
class PolyhedralSurfaceType(SurfaceType):
    patch: List[PolygonType] = field(
        default_factory=list,
        metadata={
            "name": "Patch",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "min_occurs": 1,
        },
    )


@dataclass
class ReferenceSystemType(AbstractReferenceSystemType):
    axis_name: List[str] = field(
        default_factory=list,
        metadata={
            "name": "AxisName",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "min_occurs": 1,
        },
    )


@dataclass
class TriangleType(PolygonType):
    pass


@dataclass
class TriangulatedIrregularNetworkType(PolyhedralSurfaceType):
    triangular_patch: List[TriangleType] = field(
        default_factory=list,
        metadata={
            "name": "TriangularPatch",
            "type": "Element",
            "namespace": "urn:SFA:1.2.0",
            "min_occurs": 1,
        },
    )



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

from abc import ABC, abstractmethod
from typing import Iterable, Optional

import geojson
import shapely

from .imaged_feature_property_accessor import ImagedFeaturePropertyAccessor


class Feature2DSpatialIndex(ABC):
    """
    A query-only spatial index allowing clients to lookup features using 2D geometries
    """

    @abstractmethod
    def find_intersects(self, geometry: shapely.Geometry) -> Iterable[geojson.Feature]:
        """
        Return the features intersecting the input geometry.

        :param geometry: geometry to query the index
        :return: the features
        """

    @abstractmethod
    def find_nearest(self, geometry: shapely.Geometry, max_distance: Optional[float] = None) -> Iterable[geojson.Feature]:
        """
        Return the nearest feature for the input geometry based on distance within two-dimensional Cartesian space.

        :param geometry: geometry to query the index
        :param max_distance: maximum distance
        :return: the nearest features
        """


class STRFeature2DSpatialIndex(Feature2DSpatialIndex):
    """
    Implementation of the 2D spatial index for GeoJSON features using Shapely's Sort-Tile-Recursive (STR)
    tree datastructure.
    """

    def __init__(
        self,
        feature_collection: geojson.FeatureCollection,
        use_image_geometries: bool = True,
        property_accessor: ImagedFeaturePropertyAccessor = ImagedFeaturePropertyAccessor(),
    ) -> None:
        self.use_image_geometries = use_image_geometries
        self.features = feature_collection.features
        if use_image_geometries and property_accessor is not None:
            geometries = [property_accessor.find_image_geometry(feature) for feature in self.features]
        else:
            geometries = [(shapely.shape(feature.geometry), feature) for feature in self.features]

        self.index = shapely.STRtree(geometries)

    def find_intersects(self, geometry: shapely.Geometry) -> Iterable[geojson.Feature]:
        result_indexes = self.index.query(geometry, predicate="intersects")
        return [self.features[i] for i in result_indexes]

    def find_nearest(self, geometry: shapely.Geometry, max_distance: Optional[float] = None) -> Iterable[geojson.Feature]:
        if max_distance is None:
            if self.use_image_geometries:
                max_distance = 50
            else:
                max_distance = 1.0
        result_indexes = self.index.query_nearest(geometry, max_distance=max_distance)
        return [self.features[i] for i in result_indexes]



#  Copyright 2023-2024 Amazon.com, Inc. or its affiliates.

"""This file was generated by xsdata, v23.8, on 2023-10-05 09:59:45

Generator: DataclassGenerator
See: https://xsdata.readthedocs.io/
"""
from .ic_ism import (
    LongStringWithSecurityType,
    Notice,
    NoticeBaseType,
    NoticeExternal,
    NoticeExternalList,
    NoticeExternalListType,
    NoticeExternalType,
    NoticeList,
    NoticeListType,
    NoticeText,
    NoticeType,
    ShortStringWithSecurityType,
)

__all__ = [
    "LongStringWithSecurityType",
    "Notice",
    "NoticeBaseType",
    "NoticeExternal",
    "NoticeExternalList",
    "NoticeExternalListType",
    "NoticeExternalType",
    "NoticeList",
    "NoticeListType",
    "NoticeText",
    "NoticeType",
    "ShortStringWithSecurityType",
]
